@@ -26,9 +26,7 @@ ldns_lookup_table ldns_signing_algorithms[] = {
         { LDNS_SIGN_RSASHA1_NSEC3, "RSASHA1_NSEC3" },
 #ifdef USE_SHA2
         { LDNS_SIGN_RSASHA256, "RSASHA256" },
-        { LDNS_SIGN_RSASHA256_NSEC3, "RSASHA256_NSEC3" },
         { LDNS_SIGN_RSASHA512, "RSASHA512" },
-        { LDNS_SIGN_RSASHA512_NSEC3, "RSASHA512_NSEC3" },
 #endif
         { LDNS_SIGN_DSA, "DSA" },
         { LDNS_SIGN_DSA_NSEC3, "DSA_NSEC3" },
@@ -38,7 +36,6 @@ ldns_lookup_table ldns_signing_algorithms[] = {
         { 0, NULL }
 };
 
-#ifdef HAVE_SSL
 ldns_key_list *
 ldns_key_list_new()
 {
@@ -69,7 +66,11 @@ ldns_key_new()
 		ldns_key_set_inception(newkey, 0);
 		ldns_key_set_expiration(newkey, 0);
 		ldns_key_set_pubkey_owner(newkey, NULL);
+#ifdef HAVE_SSL
+		ldns_key_set_evp_key(newkey, NULL);
+#endif /* HAVE_SSL */
 		ldns_key_set_hmac_key(newkey, NULL);
+		ldns_key_set_external_key(newkey, NULL);
 		return newkey;
 	}
 	return NULL;
@@ -89,7 +90,7 @@ ldns_key_new_frm_engine(ldns_key **key, ENGINE *e, char *key_id, ldns_algorithm
 
 	k = ldns_key_new();
 	k->_key.key = ENGINE_load_private_key(e, key_id, UI_OpenSSL(), NULL);
-	ldns_key_set_algorithm(k, alg);
+	ldns_key_set_algorithm(k, (ldns_signing_algorithm) alg);
 	if (!k->_key.key) {
 		return LDNS_STATUS_ENGINE_KEY_NOT_LOADED;
 	} else {
@@ -106,10 +107,12 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	char *d;
 	ldns_signing_algorithm alg;
 	ldns_rr *key_rr;
+#ifdef HAVE_SSL
 	RSA *rsa;
 	DSA *dsa;
 	unsigned char *hmac;
 	size_t hmac_size;
+#endif /* HAVE_SSL */
 
 	k = ldns_key_new();
 
@@ -172,29 +175,13 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 #else
 		fprintf(stderr, "Warning: SHA256 not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
-#endif
-	}
-	if (strncmp(d, "9 RSASHA256", 2) == 0) {
-#ifdef USE_SHA2
-		alg = LDNS_SIGN_RSASHA256_NSEC3;
-#else
-		fprintf(stderr, "Warning: SHA256 not compiled into this ");
-		fprintf(stderr, "version of ldns\n");
 #endif
 	}
 	if (strncmp(d, "10 RSASHA512", 3) == 0) {
 #ifdef USE_SHA2
 		alg = LDNS_SIGN_RSASHA512;
 #else
-		fprintf(stderr, "Warning: SHA256 not compiled into this ");
-		fprintf(stderr, "version of ldns\n");
-#endif
-	}
-	if (strncmp(d, "11 RSASHA512", 3) == 0) {
-#ifdef USE_SHA2
-		alg = LDNS_SIGN_RSASHA512_NSEC3;
-#else
-		fprintf(stderr, "Warning: SHA256 not compiled into this ");
+		fprintf(stderr, "Warning: SHA512 not compiled into this ");
 		fprintf(stderr, "version of ldns\n");
 #endif
 	}
@@ -216,29 +203,33 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 		case LDNS_RSASHA1_NSEC3:
 #ifdef USE_SHA2
 		case LDNS_SIGN_RSASHA256:
-		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
-		case LDNS_SIGN_RSASHA512_NSEC3:
 #endif
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			rsa = ldns_key_new_frm_fp_rsa_l(fp, line_nr);
 			ldns_key_set_rsa_key(k, rsa);
 			RSA_free(rsa);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_DSA:
 		case LDNS_DSA_NSEC3:
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			dsa = ldns_key_new_frm_fp_dsa_l(fp, line_nr);
 			ldns_key_set_dsa_key(k, dsa);
 			DSA_free(dsa);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
 			ldns_key_set_algorithm(k, alg);
+#ifdef HAVE_SSL
 			hmac = ldns_key_new_frm_fp_hmac_l(fp, line_nr, &hmac_size);
 			ldns_key_set_hmac_size(k, hmac_size);
 			ldns_key_set_hmac_key(k, hmac);
+#endif /* HAVE_SSL */
 			break;
 		case 0:
 		default:
@@ -257,6 +248,7 @@ ldns_key_new_frm_fp_l(ldns_key **key, FILE *fp, int *line_nr)
 	return LDNS_STATUS_ERR;
 }
 
+#ifdef HAVE_SSL
 RSA *
 ldns_key_new_frm_fp_rsa(FILE *f)
 {
@@ -392,6 +384,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	return rsa;
 
 error:
+	RSA_free(rsa);
 	LDNS_FREE(d);
 	LDNS_FREE(buf);
 	return NULL;
@@ -492,7 +485,7 @@ ldns_key_new_frm_fp_hmac(FILE *f, size_t *hmac_size)
 unsigned char *
 ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 {
-	int i;
+	size_t i;
 	char *d;
 	unsigned char *buf;
 
@@ -504,7 +497,9 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 	if (ldns_fget_keyword_data_l(f, "Key", ": ", d, "\n", LDNS_MAX_LINELEN, line_nr) == -1) {
 		goto error;
 	}
-	i = ldns_b64_pton((const char*)d, buf, ldns_b64_ntop_calculate_size(strlen(d)));
+	i = (size_t) ldns_b64_pton((const char*)d,
+	                           buf,
+	                           ldns_b64_ntop_calculate_size(strlen(d)));
 
 	*hmac_size = i;
 	return buf;
@@ -515,17 +510,19 @@ ldns_key_new_frm_fp_hmac_l(FILE *f, int *line_nr, size_t *hmac_size)
 	*hmac_size = 0;
 	return NULL;
 }
+#endif /* HAVE_SSL */
 
 
 ldns_key *
 ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 {
 	ldns_key *k;
+#ifdef HAVE_SSL
 	DSA *d;
 	RSA *r;
-#ifndef HAVE_SSL
+#else
 	int i;
-	uint16_t offset;
+	uint16_t offset = 0;
 #endif
 	unsigned char *hmac;
 
@@ -538,18 +535,19 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 		case LDNS_SIGN_RSASHA1:
 		case LDNS_SIGN_RSASHA1_NSEC3:
 		case LDNS_SIGN_RSASHA256:
-		case LDNS_SIGN_RSASHA256_NSEC3:
 		case LDNS_SIGN_RSASHA512:
-		case LDNS_SIGN_RSASHA512_NSEC3:
+#ifdef HAVE_SSL
 			r = RSA_generate_key((int)size, RSA_F4, NULL, NULL);
 			if (RSA_check_key(r) != 1) {
 				return NULL;
 			}
 
 			ldns_key_set_rsa_key(k, r);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_DSA:
 		case LDNS_SIGN_DSA_NSEC3:
+#ifdef HAVE_SSL
 			d = DSA_generate_parameters((int)size, NULL, 0, NULL, NULL, NULL, NULL);
 			if (!d) {
 				return NULL;
@@ -558,17 +556,20 @@ ldns_key_new_frm_algorithm(ldns_signing_algorithm alg, uint16_t size)
 				return NULL;
 			}
 			ldns_key_set_dsa_key(k, d);
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
+#ifdef HAVE_SSL
 			k->_key.key = NULL;
+#endif /* HAVE_SSL */
 			size = size / 8;
 			ldns_key_set_hmac_size(k, size);
 
 			hmac = LDNS_XMALLOC(unsigned char, size);
 #ifdef HAVE_SSL
-			if (RAND_bytes(hmac, size) != 1) {
+			if (RAND_bytes(hmac, (int) size) != 1) {
 				LDNS_FREE(hmac);
 				ldns_key_free(k);
 				return NULL;
@@ -618,6 +619,7 @@ ldns_key_set_flags(ldns_key *k, uint16_t f)
 	k->_extra.dnssec.flags = f;
 }
 
+#ifdef HAVE_SSL
 void
 ldns_key_set_evp_key(ldns_key *k, EVP_PKEY *e)
 {
@@ -639,6 +641,7 @@ ldns_key_set_dsa_key(ldns_key *k, DSA *d)
 	EVP_PKEY_set1_DSA(key, d);
 	k->_key.key  = key;
 }
+#endif /* HAVE_SSL */
 
 void
 ldns_key_set_hmac_key(ldns_key *k, unsigned char *hmac)
@@ -652,6 +655,12 @@ ldns_key_set_hmac_size(ldns_key *k, size_t hmac_size)
 	k->_key.hmac.size = hmac_size;
 }
 
+void
+ldns_key_set_external_key(ldns_key *k, void *external_key)
+{
+	k->_key.external_key = external_key;
+}
+
 void
 ldns_key_set_origttl(ldns_key *k, uint32_t t)
 {
@@ -722,6 +731,7 @@ ldns_key_use(const ldns_key *k)
 	return false;
 }
 
+#ifdef HAVE_SSL
 EVP_PKEY *
 ldns_key_evp_key(const ldns_key *k)
 {
@@ -731,25 +741,48 @@ ldns_key_evp_key(const ldns_key *k)
 RSA *
 ldns_key_rsa_key(const ldns_key *k)
 {
-	return EVP_PKEY_get1_RSA(k->_key.key);
+	if (k->_key.key) {
+		return EVP_PKEY_get1_RSA(k->_key.key);
+	} else {
+		return NULL;
+	}
 }
 
 DSA *
 ldns_key_dsa_key(const ldns_key *k)
 {
-	return EVP_PKEY_get1_DSA(k->_key.key);
+	if (k->_key.key) {
+		return EVP_PKEY_get1_DSA(k->_key.key);
+	} else {
+		return NULL;
+	}
 }
+#endif /* HAVE_SSL */
 
 unsigned char *
 ldns_key_hmac_key(const ldns_key *k)
 {
-	return k->_key.hmac.key;
+	if (k->_key.hmac.key) {
+		return k->_key.hmac.key;
+	} else {
+		return NULL;
+	}
 }
 
 size_t
 ldns_key_hmac_size(const ldns_key *k)
 {
-	return k->_key.hmac.size;
+	if (k->_key.hmac.size) {
+		return k->_key.hmac.size;
+	} else {
+		return 0;
+	}
+}
+
+void *
+ldns_key_external_key(const ldns_key *k)
+{
+	return k->_key.external_key;
 }
 
 uint32_t
@@ -855,6 +888,7 @@ ldns_key_list_pop_key(ldns_key_list *key_list)
         return pop;
 }       
 
+#ifdef HAVE_SSL
 static bool
 ldns_key_rsa2bin(unsigned char *data, RSA *k, uint16_t *size)
 {
@@ -915,6 +949,7 @@ ldns_key_dsa2bin(unsigned char *data, DSA *k, uint16_t *size)
 	*size = 21 + (*size * 3);
 	return true;
 }
+#endif /* HAVE_SSL */
 
 ldns_rr *
 ldns_key2rr(const ldns_key *k)
@@ -926,21 +961,19 @@ ldns_key2rr(const ldns_key *k)
 	 */
 	ldns_rr *pubkey;
 	ldns_rdf *keybin;
-	unsigned char *bin;
+	unsigned char *bin = NULL;
 	uint16_t size = 0;
+#ifdef HAVE_SSL
 	RSA *rsa = NULL;
 	DSA *dsa = NULL;
+#endif /* HAVE_SSL */
+	int internal_data = 0;
 
 	pubkey = ldns_rr_new();
 	if (!k) {
 		return NULL;
 	}
 
-	bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
-	if (!bin) {
-		return NULL;
-	}
-
 	switch (ldns_key_algorithm(k)) {
 	case LDNS_SIGN_HMACMD5:
 	case LDNS_SIGN_HMACSHA1:
@@ -969,58 +1002,82 @@ ldns_key2rr(const ldns_key *k)
 		case LDNS_RSASHA1:
 		case LDNS_RSASHA1_NSEC3:
 		case LDNS_RSASHA256:
-		case LDNS_RSASHA256_NSEC3:
 		case LDNS_RSASHA512:
-		case LDNS_RSASHA512_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 						  ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
+#ifdef HAVE_SSL
 			rsa =  ldns_key_rsa_key(k);
-			if (!rsa) {
-				return NULL;
-			} else {
+			if (rsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
 				if (!ldns_key_rsa2bin(bin, rsa, &size)) {
 					return NULL;
 				}
 				RSA_free(rsa);
+				internal_data = 1;
 			}
+#endif
 			size++;
 			break;
 		case LDNS_SIGN_DSA:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA));
+#ifdef HAVE_SSL
 			dsa = ldns_key_dsa_key(k);
-			if (!dsa) {
-				return NULL;
-			} else {
+			if (dsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
 				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
 					return NULL;
 				}
 				DSA_free(dsa);
+				internal_data = 1;
 			}
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_DSA_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA_NSEC3));
-			if (!ldns_key_dsa2bin(bin, ldns_key_dsa_key(k), &size)) {
-				return NULL;
+#ifdef HAVE_SSL
+			dsa = ldns_key_dsa_key(k);
+			if (dsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
+				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
+					return NULL;
+				}
+				DSA_free(dsa);
+				internal_data = 1;
 			}
+#endif /* HAVE_SSL */
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
-			/* tja */
+			bin = LDNS_XMALLOC(unsigned char, ldns_key_hmac_size(k));
+			if (!bin) {
+				return NULL;
+			}
 			ldns_rr_push_rdf(pubkey,
-                                        ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG,
-						ldns_key_algorithm(k)));
-                        size = ldns_key_hmac_size(k);
-			bin = LDNS_XREALLOC(bin, unsigned char, size);
+			                 ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG,
+			                 ldns_key_algorithm(k)));
+			size = ldns_key_hmac_size(k);
 			memcpy(bin, ldns_key_hmac_key(k), size);
+			internal_data = 1;
 			break;
 	}
 	/* fourth the key bin material */
-	keybin = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_B64, size, bin);
-	LDNS_FREE(bin);
-	ldns_rr_push_rdf(pubkey, keybin);
+	if (internal_data) {
+		keybin = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_B64, size, bin);
+		LDNS_FREE(bin);
+		ldns_rr_push_rdf(pubkey, keybin);
+	}
 	return pubkey;
 }
 
@@ -1036,28 +1093,11 @@ ldns_key_deep_free(ldns_key *key)
 	if (ldns_key_pubkey_owner(key)) {
 		ldns_rdf_deep_free(ldns_key_pubkey_owner(key));
 	}
-/*
-	switch(ldns_key_algorithm(key)) {
-	case LDNS_SIGN_RSASHA1:
-	case LDNS_SIGN_RSASHA1_NSEC3:
-	case LDNS_SIGN_RSAMD5:
-		if (ldns_key_rsa_key(key)) {
-			RSA_free(ldns_key_rsa_key(key));
-		}
-		break;
-	case LDNS_SIGN_DSA:
-	case LDNS_SIGN_DSA_NSEC3:
-		if (ldns_key_dsa_key(key)) {
-			DSA_free(ldns_key_dsa_key(key));
-		}
-		break;
-	case LDNS_SIGN_HMACMD5:
-		break;
-	}
-*/
+#ifdef HAVE_SSL
 	if (ldns_key_evp_key(key)) {
 		EVP_PKEY_free(ldns_key_evp_key(key));
 	}
+#endif /* HAVE_SSL */
 	if (ldns_key_hmac_key(key)) {
 		free(ldns_key_hmac_key(key));
 	}
@@ -1118,4 +1158,21 @@ ldns_read_anchor_file(const char *filename)
 	}
 }
 
-#endif /* HAVE_SSL */
+char *
+ldns_key_get_file_base_name(ldns_key *key)
+{
+	ldns_buffer *buffer;
+	char *file_base_name;
+	
+	buffer = ldns_buffer_new(255);
+	ldns_buffer_printf(buffer, "K");
+	(void)ldns_rdf2buffer_str_dname(buffer, ldns_key_pubkey_owner(key));
+	ldns_buffer_printf(buffer,
+	                   "+%03u+%05u",
+			   ldns_key_algorithm(key),
+			   ldns_key_keytag(key));
+	file_base_name = strdup(ldns_buffer_export(buffer));
+	ldns_buffer_free(buffer);
+	return file_base_name;
+}
+
