@@ -183,6 +183,7 @@ ldns_dnssec_create_nsec(ldns_dnssec_name *from,
 				    ldns_dnssec_name *to,
 				    ldns_rr_type nsec_type);
 
+
 /**
  * Creates NSEC3
  */
@@ -336,6 +337,7 @@ bool ldns_nsec_bitmap_covers_type(const ldns_rdf *nsec_bitmap, ldns_rr_type type
  */
 bool ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name);
 
+#ifdef HAVE_SSL
 /**
  * verify a packet 
  * \param[in] p the packet
@@ -348,6 +350,7 @@ bool ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name);
  * 
  */
 ldns_status ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o, ldns_rr_list *k, ldns_rr_list *s, ldns_rr_list *good_keys);
+#endif
 
 /**
  * chains nsec3 list
@@ -367,17 +370,6 @@ qsort_rr_compare_nsec3(const void *a, const void *b);
 void
 ldns_rr_list_sort_nsec3(ldns_rr_list *unsorted);
 
-/**
- * Adds NSEC3 records to the zone
- */
-ldns_status
-ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
-						 ldns_rr_list *new_rrs,
-						 uint8_t algorithm,
-						 uint8_t flags,
-						 uint16_t iterations,
-						 uint8_t salt_length,
-						 uint8_t *salt);
 /** 
  * Default callback function to always leave present signatures, and
  * add new ones
@@ -411,6 +403,7 @@ int ldns_dnssec_default_delete_signatures(ldns_rr *sig, void *n);
  */
 int ldns_dnssec_default_replace_signatures(ldns_rr *sig, void *n);
 
+#ifdef HAVE_SSL
 /**
  * Converts the DSA signature from ASN1 representation (RFC2459, as 
  * used by OpenSSL) to raw signature data as used in DNS (rfc2536)
@@ -435,5 +428,6 @@ ldns_status
 ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 						  const ldns_rdf *sig_rdf);
 
+#endif /* HAVE_SSL */
 
 #endif /* LDNS_DNSSEC_H */
