@@ -81,7 +81,8 @@ ldns_rr_free(ldns_rr *rr)
 }
 
 /* 
- * extra spaces are allowed
+ * trailing spaces are allowed
+ * leading spaces are not allowed
  * allow ttl to be optional
  * class is optional too
  * if ttl is missing, and default_ttl is 0, use DEF_TTL
@@ -93,9 +94,10 @@ ldns_rr_free(ldns_rr *rr)
  * or
  * miek.nl. IN MX 10 elektron.atoom.net
  */
-ldns_status
-ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns_rdf *origin, 
-		ldns_rdf **prev)
+static ldns_status
+ldns_rr_new_frm_str_internal(ldns_rr **newrr, const char *str,
+                             uint32_t default_ttl, ldns_rdf *origin, 
+		             ldns_rdf **prev, bool question)
 {
 	ldns_rr *new;
 	const ldns_rr_descriptor *desc;
@@ -134,14 +136,14 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 	new = ldns_rr_new();
 
 	owner = LDNS_XMALLOC(char, LDNS_MAX_DOMAINLEN + 1);
-	ttl = LDNS_XMALLOC(char, 21);
-	clas = LDNS_XMALLOC(char, 11);
+	ttl = LDNS_XMALLOC(char, LDNS_TTL_DATALEN);
+	clas = LDNS_XMALLOC(char, LDNS_SYNTAX_DATALEN);
 	rdata = LDNS_XMALLOC(char, LDNS_MAX_PACKETLEN + 1);
 	rr_buf = LDNS_MALLOC(ldns_buffer);
 	rd_buf = LDNS_MALLOC(ldns_buffer);
 	rd = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
 	b64 = LDNS_XMALLOC(char, LDNS_MAX_RDFLEN);
-	if (!new || !owner || !ttl || !clas || !rdata || !rr_buf || !rd_buf || !rd | !b64) {
+	if (!new || !owner || !ttl || !clas || !rdata || !rr_buf || !rd_buf || !rd || !b64 ) {
 		return LDNS_STATUS_MEM_ERR;
 	}
 	r_cnt = 0;
@@ -158,7 +160,8 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 		LDNS_FREE(rdata);
 		LDNS_FREE(rd);
 		LDNS_FREE(rd_buf);
-		ldns_buffer_free(rr_buf); 
+		LDNS_FREE(b64);
+		ldns_buffer_free(rr_buf);
 		ldns_rr_free(new);
 		return LDNS_STATUS_SYNTAX_ERR;
 	}
@@ -170,13 +173,14 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 		LDNS_FREE(rdata);
 		LDNS_FREE(rd);
 		LDNS_FREE(rd_buf);
+		LDNS_FREE(b64);
 		ldns_buffer_free(rr_buf);
 		ldns_rr_free(new);
 		return LDNS_STATUS_SYNTAX_TTL_ERR;
 	}
 	ttl_val = (uint32_t) strtol(ttl, NULL, 10);
 
-	if (strlen(ttl) > 0 && !isdigit(ttl[0])) {
+	if (strlen(ttl) > 0 && !isdigit((int) ttl[0])) {
 		/* ah, it's not there or something */
 		if (default_ttl == 0) {
 			ttl_val = LDNS_DEFAULT_TTL;
@@ -344,7 +348,9 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 				quoted = false;
 				/* if type = B64, the field may contain spaces */
 				if (ldns_rr_descriptor_field_type(desc, 
-							r_cnt) == LDNS_RDF_TYPE_B64 ||
+					    r_cnt) == LDNS_RDF_TYPE_B64 ||
+				    ldns_rr_descriptor_field_type(desc, 
+					    r_cnt) == LDNS_RDF_TYPE_HEX ||
 				    ldns_rr_descriptor_field_type(desc, 
 					    r_cnt) == LDNS_RDF_TYPE_LOC ||
 				    ldns_rr_descriptor_field_type(desc, 
@@ -377,14 +383,21 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 
 				/* because number of fields can be variable, we can't
 				   rely on _maximum() only */
+				/* skip spaces */
+				while (ldns_buffer_position(rd_buf) < ldns_buffer_limit(rd_buf) &&
+					*(ldns_buffer_current(rd_buf)) == ' ' 
+				      ) {
+					ldns_buffer_skip(rd_buf, 1);
+				}
+
 				if ((c = ldns_bget_token(rd_buf, rd, delimiters, 
-									LDNS_MAX_RDFLEN)) != -1) {
+							LDNS_MAX_RDFLEN)) != -1) {
 					/* hmmz, rfc3597 specifies that any type can be represented with
 					 * \# method, which can contain spaces...
 					 * it does specify size though... 
 					 */
 					rd_strlen = strlen(rd);
-						
+
 					/* unknown RR data */
 					if (rd_strlen == 2 && strncmp(rd, "\\#", 2) == 0 && !quoted) {
 						c = ldns_bget_token(rd_buf, rd, delimiters, LDNS_MAX_RDFLEN);
@@ -419,7 +432,8 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 							ldns_write_uint16(hex_data, hex_data_size);
 							ldns_hexstring_to_data(hex_data + 2, hex_data_str);
 							hex_pos = 0;
-							ldns_wire2rdf(new, hex_data, hex_data_size+2, &hex_pos);
+							(void) ldns_wire2rdf(new, hex_data,
+							                 hex_data_size+2, &hex_pos);
 							LDNS_FREE(hex_data);
 						} else {
 							r = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_HEX, hex_data_str);
@@ -430,11 +444,19 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 					} else {
 						/* Normal RR */
 						switch(ldns_rr_descriptor_field_type(desc, r_cnt)) {
+						case LDNS_RDF_TYPE_HEX:
 						case LDNS_RDF_TYPE_B64:
 							/* can have spaces, and will always be the last 
 							 * record of the rrdata. Read in the rest */
-							if ((c = ldns_bget_token(rd_buf, b64, "\n", LDNS_MAX_RDFLEN)) != -1) {
-								rd = strncat(rd, b64, LDNS_MAX_RDFLEN);
+							if ((c = ldns_bget_token(rd_buf,
+												b64,
+												"\n",
+												LDNS_MAX_RDFLEN))
+							    != -1) {
+								rd = strncat(rd,
+										   b64,
+										   LDNS_MAX_RDFLEN
+										   - strlen(rd) - 1);
 							}
 							r = ldns_rdf_new_frm_str(
 									ldns_rr_descriptor_field_type(desc, r_cnt),
@@ -481,7 +503,6 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 							ldns_rr_free(new);
 							return LDNS_STATUS_SYNTAX_RDATA_ERR;
 						}
-						
 					}
 					if (quoted) {
 						if (ldns_buffer_available(rd_buf, 1)) {
@@ -501,12 +522,42 @@ ldns_rr_new_frm_str(ldns_rr **newrr, const char *str, uint32_t default_ttl, ldns
 	ldns_buffer_free(rr_buf);
 	LDNS_FREE(rdata);
 
+	if (!question && desc && ldns_rr_rd_count(new) < r_min) {
+		ldns_rr_free(new);
+		return LDNS_STATUS_SYNTAX_MISSING_VALUE_ERR;
+	}
+
 	if (newrr) {
 		*newrr = new;
 	}
 	return LDNS_STATUS_OK;
 }
 
+ldns_status
+ldns_rr_new_frm_str(ldns_rr **newrr, const char *str,
+                    uint32_t default_ttl, ldns_rdf *origin, 
+		    ldns_rdf **prev)
+{
+	return ldns_rr_new_frm_str_internal(newrr,
+	                                    str,
+	                                    default_ttl,
+	                                    origin,
+	                                    prev,
+	                                    false);
+}
+
+ldns_status
+ldns_rr_new_question_frm_str(ldns_rr **newrr, const char *str,
+                             ldns_rdf *origin, ldns_rdf **prev)
+{
+	return ldns_rr_new_frm_str_internal(newrr,
+	                                    str,
+	                                    0,
+	                                    origin,
+	                                    prev,
+	                                    true);
+}
+
 ldns_status
 ldns_rr_new_frm_fp(ldns_rr **newrr, FILE *fp, uint32_t *ttl, ldns_rdf **origin, ldns_rdf **prev)
 {
@@ -516,7 +567,7 @@ ldns_rr_new_frm_fp(ldns_rr **newrr, FILE *fp, uint32_t *ttl, ldns_rdf **origin,
 ldns_status
 ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf **origin, ldns_rdf **prev, int *line_nr)
 {
-        char *line;
+	char *line;
 	const char *endptr;  /* unused */
 	ldns_rr *rr;
 	char *keyword;
@@ -532,20 +583,20 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 		ttl = 0;
 	}
 
-        line = LDNS_XMALLOC(char, LDNS_MAX_LINELEN + 1);
-        if (!line) {
-                return LDNS_STATUS_MEM_ERR;
-        }
+	line = LDNS_XMALLOC(char, LDNS_MAX_LINELEN + 1);
+	if (!line) {
+		return LDNS_STATUS_MEM_ERR;
+	}
 
-        /* read an entire line in from the file */
-        if ((size = ldns_fget_token_l(fp, line, LDNS_PARSE_SKIP_SPACE, LDNS_MAX_LINELEN, line_nr)) == -1) {
+	/* read an entire line in from the file */
+	if ((size = ldns_fget_token_l(fp, line, LDNS_PARSE_SKIP_SPACE, LDNS_MAX_LINELEN, line_nr)) == -1) {
 		LDNS_FREE(line);
 		/* if last line was empty, we are now at feof, which is not
 		 * always a parse error (happens when for instance last line
 		 * was a comment)
 		 */
-                return LDNS_STATUS_SYNTAX_ERR;
-        }
+		return LDNS_STATUS_SYNTAX_ERR;
+	}
 
 	/* we can have the situation, where we've read ok, but still got
 	 * no bytes to play with, in this case size is 0 
@@ -567,7 +618,7 @@ ldns_rr_new_frm_fp_l(ldns_rr **newrr, FILE *fp, uint32_t *default_ttl, ldns_rdf
 			return LDNS_STATUS_SYNTAX_DNAME_ERR;
 		}
 		*origin = tmp;
-		s = LDNS_STATUS_SYNTAX_ORIGIN;		
+		s = LDNS_STATUS_SYNTAX_ORIGIN;
 	} else if ((keyword = strstr(line, "$TTL "))) {
 		if (default_ttl) {
 			*default_ttl = ldns_str2period(keyword + 5, &endptr);
@@ -672,7 +723,7 @@ ldns_rr_pop_rdf(ldns_rr *rr)
 		return NULL;
 	}
 
-	pop = rr->_rdata_fields[rd_count];
+	pop = rr->_rdata_fields[rd_count - 1];
 	
 	/* shrink the array */
 	rr->_rdata_fields = LDNS_XREALLOC(
@@ -884,6 +935,7 @@ ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t pos)
 			pos);
 		if (!list_rdf) {
 			/* pos is too large or any other error */
+			ldns_rr_list_deep_free(subtyped);
 			return NULL;
 		}
 
@@ -897,6 +949,7 @@ ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t pos)
 	if (ldns_rr_list_rr_count(subtyped) > 0) {
 		return subtyped;
 	} else {
+		ldns_rr_list_free(subtyped);
 		return NULL;
 	}
 }
@@ -1031,7 +1084,7 @@ ldns_is_rrset(ldns_rr_list *rr_list)
 	ldns_rr *tmp;
 	size_t i;
 	
-	if (!rr_list) {
+	if (!rr_list || ldns_rr_list_rr_count(rr_list) == 0) {
 		return false;
 	}
 
@@ -1043,7 +1096,7 @@ ldns_is_rrset(ldns_rr_list *rr_list)
 
 	/* compare these with the rest of the rr_list, start with 1 */
 	for (i = 1; i < ldns_rr_list_rr_count(rr_list); i++) {
-		tmp = ldns_rr_list_rr(rr_list, 1);
+		tmp = ldns_rr_list_rr(rr_list, i);
 		if (t != ldns_rr_get_type(tmp)) {
 			return false;
 		}
@@ -1214,7 +1267,7 @@ ldns_rr_list_clone(const ldns_rr_list *rrlist)
 		    );
 		if (!r) {
 			/* huh, failure in cloning */
-			ldns_rr_list_free(new_list);
+			ldns_rr_list_deep_free(new_list);
 			return NULL;
 		}
 		ldns_rr_list_push_rr(new_list, r);
@@ -1380,7 +1433,7 @@ int ldns_rr_compare_wire(ldns_buffer *rr1_buf, ldns_buffer *rr2_buf)
         /* jump to rdata section (PAST the rdata length field, otherwise
            rrs with different lengths might be sorted erroneously */
         offset += 11;
-	min_len = (rr1_len < rr2_len) ? rr1_len : rr2_len;
+	   min_len = (rr1_len < rr2_len) ? rr1_len : rr2_len;
         /* Compare RRs RDATA byte for byte. */
         for(i = offset; i < min_len; i++) {
                 if (*ldns_buffer_at(rr1_buf,i) < *ldns_buffer_at(rr2_buf,i)) {
@@ -1418,12 +1471,18 @@ ldns_rr_compare(const ldns_rr *rr1, const ldns_rr *rr2)
 		rr1_buf = ldns_buffer_new(rr1_len);
 		rr2_buf = ldns_buffer_new(rr2_len);
 
-		if (ldns_rr2buffer_wire(rr1_buf, rr1, LDNS_SECTION_ANY) != LDNS_STATUS_OK) {
+		if (ldns_rr2buffer_wire_canonical(rr1_buf,
+								    rr1,
+								    LDNS_SECTION_ANY) 
+		    != LDNS_STATUS_OK) {
 			ldns_buffer_free(rr1_buf);
 			ldns_buffer_free(rr2_buf);
 			return 0; 
 		}
-		if (ldns_rr2buffer_wire(rr2_buf, rr2, LDNS_SECTION_ANY) != LDNS_STATUS_OK) {
+		if (ldns_rr2buffer_wire_canonical(rr2_buf,
+								    rr2,
+								    LDNS_SECTION_ANY)
+		    != LDNS_STATUS_OK) {
 			ldns_buffer_free(rr1_buf);
 			ldns_buffer_free(rr2_buf);
 			return 0;
@@ -1442,9 +1501,7 @@ bool
 ldns_rr_compare_ds(const ldns_rr *orr1, const ldns_rr *orr2)
 {
 	bool result;
-#ifdef HAVE_SSL
 	ldns_rr *ds_repr;
-#endif /* HAVE_SSL */
 	ldns_rr *rr1 = ldns_rr_clone(orr1);
 	ldns_rr *rr2 = ldns_rr_clone(orr2);
 	
@@ -1452,7 +1509,6 @@ ldns_rr_compare_ds(const ldns_rr *orr1, const ldns_rr *orr2)
 	ldns_rr_set_ttl(rr1, 0);
 	ldns_rr_set_ttl(rr2, 0);
 
-#ifdef HAVE_SSL
 	if (ldns_rr_get_type(rr1) == LDNS_RR_TYPE_DS &&
 	    ldns_rr_get_type(rr2) == LDNS_RR_TYPE_DNSKEY) {
 	    	ds_repr = ldns_key_rr2ds(rr2, LDNS_SHA1);
@@ -1467,9 +1523,6 @@ ldns_rr_compare_ds(const ldns_rr *orr1, const ldns_rr *orr2)
 	} else {
 		result = (ldns_rr_compare(rr1, rr2) == 0);
 	}	
-#else
-		result = (ldns_rr_compare(rr1, rr2) == 0);
-#endif /* HAVE_SSL */
 	
 	ldns_rr_free(rr1);
 	ldns_rr_free(rr2);
@@ -1678,7 +1731,7 @@ static const ldns_rdf_type type_kx_wireformat[] = {
 static const ldns_rdf_type type_cert_wireformat[] = {
 	 LDNS_RDF_TYPE_CERT_ALG, LDNS_RDF_TYPE_INT16, LDNS_RDF_TYPE_ALG, LDNS_RDF_TYPE_B64
 };
-static const ldns_rdf_type type_a6_wireformat[] = { LDNS_RDF_TYPE_DNAME };
+static const ldns_rdf_type type_a6_wireformat[] = { LDNS_RDF_TYPE_UNKNOWN };
 static const ldns_rdf_type type_dname_wireformat[] = { LDNS_RDF_TYPE_DNAME };
 static const ldns_rdf_type type_sink_wireformat[] = { LDNS_RDF_TYPE_INT8,
 	LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_INT8, LDNS_RDF_TYPE_B64
@@ -1819,7 +1872,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 	/* 37 */
 	{LDNS_RR_TYPE_CERT, "CERT", 4, 4, type_cert_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 38 */
-	{LDNS_RR_TYPE_A6, "A6", 1, 1, type_a6_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 1 },
+	{LDNS_RR_TYPE_A6, "A6", 1, 1, type_a6_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 39 */
 	{LDNS_RR_TYPE_DNAME, "DNAME", 1, 1, type_dname_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 1 },
 	/* 40 */
@@ -1837,13 +1890,13 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 	/* 46 */
 	{LDNS_RR_TYPE_RRSIG, "RRSIG", 9, 9, type_rrsig_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 1 },
 	/* 47 */
-	{LDNS_RR_TYPE_NSEC, "NSEC", 2, 2, type_nsec_wireformat, LDNS_RDF_TYPE_NSEC, LDNS_RR_NO_COMPRESS, 1 },
+	{LDNS_RR_TYPE_NSEC, "NSEC", 1, 2, type_nsec_wireformat, LDNS_RDF_TYPE_NSEC, LDNS_RR_NO_COMPRESS, 1 },
 	/* 48 */
 	{LDNS_RR_TYPE_DNSKEY, "DNSKEY", 4, 4, type_dnskey_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 49 */
 {LDNS_RR_TYPE_DHCID, "DHCID", 1, 1, type_dhcid_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 50 */
-{LDNS_RR_TYPE_NSEC3, "NSEC3", 6, 6, type_nsec3_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+	{LDNS_RR_TYPE_NSEC3, "NSEC3", 5, 6, type_nsec3_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 51 */
 {LDNS_RR_TYPE_NSEC3PARAMS, "NSEC3PARAM", 4, 4, type_nsec3params_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 	/* 52 */
@@ -1894,7 +1947,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE96", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE97", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE98", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
-{LDNS_RR_TYPE_NULL, "TYPE99", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+{LDNS_RR_TYPE_SPF,  "SPF", 1, 0, NULL, LDNS_RDF_TYPE_STR, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE100", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE101", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE102", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
@@ -2045,7 +2098,9 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE247", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE248", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE249", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
-{LDNS_RR_TYPE_TSIG, "TSIG", 8, 9, type_tsig_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 }
+{LDNS_RR_TYPE_TSIG, "TSIG", 8, 9, type_tsig_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+/* split in array, no longer contiguous */
+{LDNS_RR_TYPE_DLV, "DLV", 4, 4, type_ds_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 }
 };
 /** \endcond */
 
@@ -2063,7 +2118,7 @@ ldns_rr_descript(uint16_t type)
 	if (type <= LDNS_RDATA_FIELD_DESCRIPTORS_COMMON) {
 		return &rdata_field_descriptors[type];
 	} else {
-	        /* TODO: tmp until type code */
+		/* because not all array index equals type code */
 		for (i = LDNS_RDATA_FIELD_DESCRIPTORS_COMMON;
 		     i < LDNS_RDATA_FIELD_DESCRIPTORS_COUNT;
 		     i++) {
@@ -2133,7 +2188,8 @@ ldns_get_rr_type_by_name(const char *name)
 		if(desc_name &&
 		   strlen(name) == strlen(desc_name) &&
 		   strncasecmp(name, desc_name, strlen(desc_name)) == 0) {
-			return i;
+			/* because not all array index equals type code */
+			return desc->_type;
 		}
 	}
 	
@@ -2189,3 +2245,24 @@ ldns_rdf2rr_type(const ldns_rdf *rd)
         r = (ldns_rr_type) ldns_rdf2native_int16(rd);
         return r;
 }
+
+ldns_rr_type
+ldns_rr_list_type(const ldns_rr_list *rr_list)
+{
+	if (rr_list && ldns_rr_list_rr_count(rr_list) > 0) {
+		return ldns_rr_get_type(ldns_rr_list_rr(rr_list, 0));
+	} else {
+		return 0;
+	}
+}
+
+ldns_rdf *
+ldns_rr_list_owner(const ldns_rr_list *rr_list)
+{
+	if (rr_list && ldns_rr_list_rr_count(rr_list) > 0) {
+		return ldns_rr_owner(ldns_rr_list_rr(rr_list, 0));
+	} else {
+		return NULL;
+	}
+}
+
