@@ -360,7 +360,7 @@ service_send(struct ringbuf* ring, struct timeval* now, ldns_buffer* pkt,
 			(unsigned)tv.tv_sec, (unsigned)tv.tv_usec);
 		log_addr(1, "from client", &p->addr, p->addr_len);
 		/* send it */
-		sent = sendto(p->s, ldns_buffer_begin(pkt), 
+		sent = sendto(p->s, (void*)ldns_buffer_begin(pkt), 
 			ldns_buffer_limit(pkt), 0, 
 			(struct sockaddr*)srv_addr, srv_len);
 		if(sent == -1) {
@@ -384,7 +384,7 @@ do_proxy(struct proxy* p, int retsock, ldns_buffer* pkt)
 	int i;
 	ssize_t r;
 	for(i=0; i<TRIES_PER_SELECT; i++) {
-		r = recv(p->s, ldns_buffer_begin(pkt), 
+		r = recv(p->s, (void*)ldns_buffer_begin(pkt), 
 			ldns_buffer_capacity(pkt), 0);
 		if(r == -1) {
 #ifndef USE_WINSOCK
@@ -403,8 +403,8 @@ do_proxy(struct proxy* p, int retsock, ldns_buffer* pkt)
 		log_addr(1, "return reply to client", &p->addr, p->addr_len);
 		/* send reply back to the real client */
 		p->numreturn++;
-		r = sendto(retsock, ldns_buffer_begin(pkt), (size_t)r, 0,
-			(struct sockaddr*)&p->addr, p->addr_len);
+		r = sendto(retsock, (void*)ldns_buffer_begin(pkt), (size_t)r, 
+			0, (struct sockaddr*)&p->addr, p->addr_len);
 		if(r == -1) {
 #ifndef USE_WINSOCK
 			log_err("sendto: %s", strerror(errno));
@@ -492,7 +492,7 @@ service_recv(int s, struct ringbuf* ring, ldns_buffer* pkt,
 	struct proxy* p;
 	for(i=0; i<TRIES_PER_SELECT; i++) {
 		from_len = (socklen_t)sizeof(from);
-		len = recvfrom(s, ldns_buffer_begin(pkt),
+		len = recvfrom(s, (void*)ldns_buffer_begin(pkt),
 			ldns_buffer_capacity(pkt), 0,
 			(struct sockaddr*)&from, &from_len);
 		if(len < 0) {
@@ -542,9 +542,15 @@ tcp_proxy_delete(struct tcp_proxy* p)
 		free(s);
 		s = sn;
 	}
+#ifndef USE_WINSOCK
 	close(p->client_s);
 	if(p->server_s != -1)
 		close(p->server_s);
+#else
+	closesocket(p->client_s);
+	if(p->server_s != -1)
+		closesocket(p->server_s);
+#endif
 	free(p);
 }
 
@@ -593,14 +599,16 @@ service_tcp_listen(int s, fd_set* rorig, int* max, struct tcp_proxy** proxies,
 #ifndef USE_WINSOCK
 		if(errno != EINPROGRESS) {
 			log_err("tcp connect: %s", strerror(errno));
+			close(p->server_s);
+			close(p->client_s);
 #else
 		if(WSAGetLastError() != WSAEWOULDBLOCK &&
 			WSAGetLastError() != WSAEINPROGRESS) {
 			log_err("tcp connect: %s", 
 				wsa_strerror(WSAGetLastError()));
+			closesocket(p->server_s);
+			closesocket(p->client_s);
 #endif
-			close(p->server_s);
-			close(p->client_s);
 			free(p);
 			return;
 		}
@@ -628,7 +636,7 @@ tcp_relay_read(int s, struct tcp_send_list** first,
 	struct timeval* delay, ldns_buffer* pkt)
 {
 	struct tcp_send_list* item;
-	ssize_t r = recv(s, ldns_buffer_begin(pkt), 
+	ssize_t r = recv(s, (void*)ldns_buffer_begin(pkt), 
 		ldns_buffer_capacity(pkt), 0);
 	if(r == -1) {
 #ifndef USE_WINSOCK
@@ -753,7 +761,11 @@ service_tcp_relay(struct tcp_proxy** tcp_proxies, struct timeval* now,
 			log_addr(1, "read tcp answer", &p->addr, p->addr_len);
 			if(!tcp_relay_read(p->server_s, &p->answerlist, 
 				&p->answerlast, now, delay, pkt)) {
+#ifndef USE_WINSOCK
 				close(p->server_s);
+#else
+				closesocket(p->server_s);
+#endif
 				FD_CLR(FD_SET_T p->server_s, worig);
 				FD_CLR(FD_SET_T p->server_s, rorig);
 				p->server_s = -1;
@@ -881,7 +893,11 @@ proxy_list_clear(struct proxy* p)
 			"%u returned\n", i++, from, port, (int)p->numreuse+1,
 			(unsigned)p->numwait, (unsigned)p->numsent, 
 			(unsigned)p->numreturn);
+#ifndef USE_WINSOCK
 		close(p->s);
+#else
+		closesocket(p->s);
+#endif
 		free(p);
 		p = np;
 	}
@@ -1083,8 +1099,13 @@ service(const char* bind_str, int bindport, const char* serv_str,
 
 	/* cleanup */
 	verbose(1, "cleanup");
+#ifndef USE_WINSOCK
 	close(s);
 	close(listen_s);
+#else
+	closesocket(s);
+	closesocket(listen_s);
+#endif
 	ldns_buffer_free(pkt);
 	ring_delete(ring);
 }
