@@ -474,14 +474,16 @@ verify_test()
 	verifytest_file("testdata/test_signatures.6", "20080416005004");
 	verifytest_file("testdata/test_signatures.7", "20070829144150");
 	verifytest_file("testdata/test_signatures.8", "20070829144150");
-#ifdef HAVE_EVP_SHA256
+#if defined(HAVE_EVP_SHA256) && defined(USE_SHA2)
 	verifytest_file("testdata/test_signatures.9", "20070829144150");
 	verifytest_file("testdata/test_signatures.11", "20070829144150");
+	verifytest_file("testdata/test_signatures.14", "20090101000000");
 #endif
-#ifdef HAVE_EVP_SHA512
+#if defined(HAVE_EVP_SHA512) && defined(USE_SHA2)
 	verifytest_file("testdata/test_signatures.10", "20070829144150");
 #endif
 	verifytest_file("testdata/test_signatures.12", "20090107100022");
+	verifytest_file("testdata/test_signatures.13", "20080414005004");
 	dstest_file("testdata/test_ds_sig.1");
 	nsectest();
 	nsec3_hash_test("testdata/test_nsec3_hash.1");
