@@ -38,7 +38,7 @@
  *
  * This file contains event notification functions.
  */
-
+#include "config.h"
 #include "util/netevent.h"
 #include "util/log.h"
 #include "util/net_help.h"
@@ -160,7 +160,11 @@ comm_base_create(int sigs)
 		b->eb->base=(struct event_base *)ev_loop_new(EVFLAG_AUTO);
 #  else
 	(void)sigs;
+#    ifdef HAVE_EVENT_BASE_NEW
+	b->eb->base = event_base_new();
+#    else
 	b->eb->base = event_init();
+#    endif
 #  endif
 #endif
 	if(!b->eb->base) {
@@ -242,14 +246,37 @@ comm_point_send_udp_msg(struct comm_point *c, ldns_buffer* packet,
 		log_err("error: send empty UDP packet");
 #endif
 	log_assert(addr && addrlen > 0);
-	sent = sendto(c->fd, ldns_buffer_begin(packet), 
+	sent = sendto(c->fd, (void*)ldns_buffer_begin(packet), 
 		ldns_buffer_remaining(packet), 0,
 		addr, addrlen);
 	if(sent == -1) {
-#ifdef ENETUNREACH
-		if(errno == ENETUNREACH && verbosity < VERB_ALGO)
-			return 0;
+		/* do not log transient errors (unless high verbosity) */
+#if defined(ENETUNREACH) || defined(EHOSTDOWN) || defined(EHOSTUNREACH) || defined(ENETDOWN)
+		switch(errno) {
+#  ifdef ENETUNREACH
+			case ENETUNREACH:
+#  endif
+#  ifdef EHOSTDOWN
+			case EHOSTDOWN:
+#  endif
+#  ifdef EHOSTUNREACH
+			case EHOSTUNREACH:
+#  endif
+#  ifdef ENETDOWN
+			case ENETDOWN:
+#  endif
+				if(verbosity < VERB_ALGO)
+					return 0;
+			default:
+				break;
+		}
 #endif
+		/* squelch errors where people deploy AAAA ::ffff:bla for
+		 * authority servers, which we try for intranets. */
+		if(errno == EINVAL && addr_is_ip4mapped(
+			(struct sockaddr_storage*)addr, addrlen) &&
+			verbosity < VERB_DETAIL)
+			return 0;
 #ifndef USE_WINSOCK
 		verbose(VERB_OPS, "sendto failed: %s", strerror(errno));
 #else
@@ -520,7 +547,7 @@ comm_point_udp_callback(int fd, short event, void* arg)
 		rep.addrlen = (socklen_t)sizeof(rep.addr);
 		log_assert(fd != -1);
 		log_assert(ldns_buffer_remaining(rep.c->buffer) > 0);
-		recv = recvfrom(fd, ldns_buffer_begin(rep.c->buffer), 
+		recv = recvfrom(fd, (void*)ldns_buffer_begin(rep.c->buffer), 
 			ldns_buffer_remaining(rep.c->buffer), 0, 
 			(struct sockaddr*)&rep.addr, &rep.addrlen);
 		if(recv == -1) {
@@ -701,7 +728,7 @@ comm_point_tcp_handle_read(int fd, struct comm_point* c, int short_ok)
 	log_assert(fd != -1);
 	if(c->tcp_byte_count < sizeof(uint16_t)) {
 		/* read length bytes */
-		r = recv(fd, ldns_buffer_at(c->buffer, c->tcp_byte_count), 
+		r = recv(fd,(void*)ldns_buffer_at(c->buffer,c->tcp_byte_count),
 			sizeof(uint16_t)-c->tcp_byte_count, 0);
 		if(r == 0)
 			return 0;
@@ -750,7 +777,7 @@ comm_point_tcp_handle_read(int fd, struct comm_point* c, int short_ok)
 	}
 
 	log_assert(ldns_buffer_remaining(c->buffer) > 0);
-	r = recv(fd, ldns_buffer_current(c->buffer), 
+	r = recv(fd, (void*)ldns_buffer_current(c->buffer), 
 		ldns_buffer_remaining(c->buffer), 0);
 	if(r == 0) {
 		return 0;
@@ -866,6 +893,8 @@ comm_point_tcp_handle_write(int fd, struct comm_point* c)
 				return 1;
 			log_err("tcp writev: %s", strerror(errno));
 #else
+			if(WSAGetLastError() == WSAENOTCONN)
+				return 1;
 			if(WSAGetLastError() == WSAEINPROGRESS)
 				return 1;
 			if(WSAGetLastError() == WSAEWOULDBLOCK) {
@@ -890,7 +919,7 @@ comm_point_tcp_handle_write(int fd, struct comm_point* c)
 		}
 	}
 	log_assert(ldns_buffer_remaining(c->buffer) > 0);
-	r = send(fd, ldns_buffer_current(c->buffer), 
+	r = send(fd, (void*)ldns_buffer_current(c->buffer), 
 		ldns_buffer_remaining(c->buffer), 0);
 	if(r == -1) {
 #ifndef USE_WINSOCK
@@ -1385,7 +1414,11 @@ comm_point_close(struct comm_point* c)
 	/* close fd after removing from event lists, or epoll.. is messed up */
 	if(c->fd != -1 && !c->do_not_close) {
 		verbose(VERB_ALGO, "close fd %d", c->fd);
+#ifndef USE_WINSOCK
 		close(c->fd);
+#else
+		closesocket(c->fd);
+#endif
 	}
 	c->fd = -1;
 }
@@ -1485,8 +1518,13 @@ comm_point_start_listening(struct comm_point* c, int newfd, int sec)
 		else	c->ev->ev.ev_events |= EV_WRITE;
 	}
 	if(newfd != -1) {
-		if(c->fd != -1)
+		if(c->fd != -1) {
+#ifndef USE_WINSOCK
 			close(c->fd);
+#else
+			closesocket(c->fd);
+#endif
+		}
 		c->fd = newfd;
 		c->ev->ev.ev_fd = c->fd;
 	}
