@@ -40,6 +40,7 @@
  */
 #include "config.h"
 #include "validator/val_anchor.h"
+#include "validator/val_sigcrypt.h"
 #include "util/data/packed_rrset.h"
 #include "util/data/dname.h"
 #include "util/log.h"
@@ -205,6 +206,7 @@ anchor_new_ta_key(struct val_anchors* anchors, uint8_t* rdata, size_t rdata_len,
  * @param type: type or RR
  * @param dclass: class of RR
  * @param rdata: rdata wireformat, starting with rdlength.
+ *	If NULL, nothing is stored, but an entry is created.
  * @param rdata_len: length of rdata including rdlength.
  * @return: NULL on error, else the trust anchor.
  */
@@ -228,6 +230,8 @@ anchor_store_new_key(struct val_anchors* anchors, uint8_t* name, uint16_t type,
 		if(!ta)
 			return NULL;
 	}
+	if(!rdata)
+		return ta;
 	/* look for duplicates */
 	if(anchor_find_key(ta, rdata, rdata_len, type)) {
 		return ta;
@@ -280,6 +284,27 @@ anchor_store_new_rr(struct val_anchors* anchors, ldns_buffer* buffer,
 	return ta;
 }
 
+/**
+ * Insert insecure anchor
+ * @param anchors: anchor storage.
+ * @param str: the domain name.
+ * @return NULL on error, Else last trust anchor point
+ */
+static struct trust_anchor*
+anchor_insert_insecure(struct val_anchors* anchors, const char* str)
+{
+	struct trust_anchor* ta;
+	ldns_rdf* nm = ldns_dname_new_frm_str(str);
+	if(!nm) {
+		log_err("parse error in domain name '%s'", str);
+		return NULL;
+	}
+	ta = anchor_store_new_key(anchors, ldns_rdf_data(nm), LDNS_RR_TYPE_DS,
+		LDNS_RR_CLASS_IN, NULL, 0);
+	ldns_rdf_deep_free(nm);
+	return ta;
+}
+
 struct trust_anchor*
 anchor_store_str(struct val_anchors* anchors, ldns_buffer* buffer,
 	const char* str)
@@ -819,6 +844,39 @@ anchors_assemble(struct val_anchors* anchors, struct trust_anchor* ta)
 	return 1;
 }
 
+/**
+ * Check DS algos for support, warn if not.
+ * @param ta: trust anchor
+ * @return number of DS anchors with unsupported algorithms.
+ */
+static size_t
+anchors_ds_unsupported(struct trust_anchor* ta)
+{
+	size_t i, num = 0;
+	for(i=0; i<ta->numDS; i++) {
+		if(!ds_digest_algo_is_supported(ta->ds_rrset, i) || 
+			!ds_key_algo_is_supported(ta->ds_rrset, i))
+			num++;
+	}
+	return num;
+}
+
+/**
+ * Check DNSKEY algos for support, warn if not.
+ * @param ta: trust anchor
+ * @return number of DNSKEY anchors with unsupported algorithms.
+ */
+static size_t
+anchors_dnskey_unsupported(struct trust_anchor* ta)
+{
+	size_t i, num = 0;
+	for(i=0; i<ta->numDNSKEY; i++) {
+		if(!dnskey_algo_is_supported(ta->dnskey_rrset, i))
+			num++;
+	}
+	return num;
+}
+
 /**
  * Assemble the rrsets in the anchors, ready for use by validator.
  * @param anchors: trust anchor storage.
@@ -828,11 +886,40 @@ static int
 anchors_assemble_rrsets(struct val_anchors* anchors)
 {
 	struct trust_anchor* ta;
-	RBTREE_FOR(ta, struct trust_anchor*, anchors->tree) {
+	struct trust_anchor* next;
+	size_t nods, nokey;
+	ta=(struct trust_anchor*)rbtree_first(anchors->tree);
+	while((rbnode_t*)ta != RBTREE_NULL) {
+		next = (struct trust_anchor*)rbtree_next(&ta->node);
+		if(ta->numDS == 0 && ta->numDNSKEY == 0) {
+			ta = next; /* skip unsigned entries, nothing to do */
+			continue;
+		}
 		if(!anchors_assemble(anchors, ta)) {
 			log_err("out of memory");
 			return 0;
 		}
+		nods = anchors_ds_unsupported(ta);
+		nokey = anchors_dnskey_unsupported(ta);
+		if(nods) {
+			log_nametypeclass(0, "warning: unsupported "
+				"algorithm for trust anchor", 
+				ta->name, LDNS_RR_TYPE_DS, ta->dclass);
+		}
+		if(nokey) {
+			log_nametypeclass(0, "warning: unsupported "
+				"algorithm for trust anchor", 
+				ta->name, LDNS_RR_TYPE_DNSKEY, ta->dclass);
+		}
+		if(nods == ta->numDS && nokey == ta->numDNSKEY) {
+			char b[257];
+			dname_str(ta->name, b);
+			log_warn("trust anchor %s has no supported algorithms,"
+				" the anchor is ignored (check if you need to"
+				" upgrade unbound and openssl)", b);
+			(void)rbtree_delete(anchors->tree, &ta->node);
+		}
+		ta = next;
 	}
 	return 1;
 }
@@ -843,6 +930,15 @@ anchors_apply_cfg(struct val_anchors* anchors, struct config_file* cfg)
 	struct config_strlist* f;
 	char* nm;
 	ldns_buffer* parsebuf = ldns_buffer_new(65535);
+	for(f = cfg->domain_insecure; f; f = f->next) {
+		if(!f->str || f->str[0] == 0) /* empty "" */
+			continue;
+		if(!anchor_insert_insecure(anchors, f->str)) {
+			log_err("error in domain-insecure: %s", f->str);
+			ldns_buffer_free(parsebuf);
+			return 0;
+		}
+	}
 	for(f = cfg->trust_anchor_file_list; f; f = f->next) {
 		if(!f->str || f->str[0] == 0) /* empty "" */
 			continue;
@@ -901,8 +997,9 @@ anchors_apply_cfg(struct val_anchors* anchors, struct config_file* cfg)
 			return 0;
 		}
 	}
-	init_parents(anchors);
+	/* first assemble, since it may delete useless anchors */
 	anchors_assemble_rrsets(anchors);
+	init_parents(anchors);
 	ldns_buffer_free(parsebuf);
 	return 1;
 }
